@@ -9,19 +9,18 @@
 #' transition on a log scale (as if we were operating with musical notes rather
 #' than frequencies in Hz). Pitch plots have two Y axes: one showing Hz and the
 #' other showing musical notation.
-#' @param anchors a dataframe of anchors with specified time and amplitude.
-#'   \code{achors$time} can be in ms (with len=NULL) or in arbitrary units, eg 0
-#'   to 1 (with duration determined by len, which must then be provided in ms).
-#'   So anchors$time is assumed to be in ms if len=NULL and relative if len is
-#'   specified. \code{anchors$value} can be on any scale.
+#' @param anchors a numeric vector of values or a list/dataframe with one column
+#'   (value) or two columns (time and value). \code{achors$time} can be in ms
+#'   (with len=NULL) or in arbitrary units, eg 0 to 1 (with duration determined
+#'   by len, which must then be provided in ms). So anchors$time is assumed to
+#'   be in ms if len=NULL and relative if len is specified. \code{anchors$value}
+#'   can be on any scale.
 #' @param len the required length of the output contour. If NULL, it will be
 #'   calculated based on the maximum time value (in ms) and \code{samplingRate}
 #' @param thisIsPitch (boolean) is this a pitch contour? If true, log-transforms
 #'   before smoothing and plots in both Hz and musical notation
-#' @param method smoothing method: 'spline' or 'loess' (default). Consider
-#'   'spline' if you have a lot of anchors and not much patience, since it is
-#'   much faster than 'loess', but it may produce weird results when there are
-#'   only a few anchors
+#' @param normalizeTime if TRUE, normalizes anchors$time values to range from 0 to 1
+#' @inheritParams soundgen
 #' @param valueFloor,valueCeiling lower/upper bounds for the contour
 #' @param plot (boolean) produce a plot?
 #' @param samplingRate sampling rate used to convert time values to points (Hz)
@@ -35,6 +34,7 @@
 #' # long format: anchors are a dataframe
 #' a = getSmoothContour(anchors = data.frame(
 #'   time = c(50, 137, 300), value = c(0.03, 0.78, 0.5)),
+#'   normalizeTime = FALSE,
 #'   voiced = 200, valueFloor = 0, plot = TRUE, main = '',
 #'   samplingRate = 16000) # breathing
 #'
@@ -50,10 +50,30 @@
 #' # two pitch anchors give loglinear F0 change
 #' a = getSmoothContour(anchors = c(220, 440),
 #'   len = 500, thisIsPitch = TRUE, plot = TRUE, samplingRate = 500)
+#'
+#' ## Two closely spaced anchors produce a pitch jump
+#' # one loess for the entire contour
+#' a1 = getSmoothContour(anchors = list(time = c(0, .15, .2, .7, 1),
+#'     value = c(360, 116, 550, 700, 610)), len = 500, thisIsPitch = TRUE,
+#'     plot = TRUE, samplingRate = 500)
+#' # two segments with a linear transition
+#' a2 = getSmoothContour(anchors = list(time = c(0, .15, .17, .7, 1),
+#'     value = c(360, 116, 550, 700, 610)), len = 500, thisIsPitch = TRUE,
+#'     plot = TRUE, samplingRate = 500)
+#' # two segments with an abrupt jump
+#' a3 = getSmoothContour(anchors = list(time = c(0, .15, .155, .7, 1),
+#'     value = c(360, 116, 550, 700, 610)), len = 500, thisIsPitch = TRUE,
+#'     plot = TRUE, samplingRate = 500)
+#' # compare:
+#' plot(a2)
+#' plot(a3)  # NB: the segment before the jump is upsampled to compensate
 getSmoothContour = function(anchors = data.frame(time = c(0, 1), value = c(0, 1)),
                             len = NULL,
                             thisIsPitch = FALSE,
-                            method = c('spline', 'loess')[2],
+                            normalizeTime = TRUE,
+                            interpol = c('approx', 'spline', 'loess')[3],
+                            discontThres = .05,
+                            jumpThres = .01,
                             valueFloor = NULL,
                             valueCeiling = NULL,
                             plot = FALSE,
@@ -64,20 +84,12 @@ getSmoothContour = function(anchors = data.frame(time = c(0, 1), value = c(0, 1)
                             voiced = NULL,
                             contourLabel = NULL,
                             ...) {
-  anchors = reformatAnchors(anchors)
-  if (is.list(anchors) && nrow(anchors) > 10 && method == 'loess') {
-    method = 'spline'
+  anchors = reformatAnchors(anchors, normalizeTime = normalizeTime)
+  if (is.list(anchors) && nrow(anchors) > 10 && nrow(anchors) < 50 && interpol == 'loess') {
+    interpol = 'spline'
     # warning('More than 10 anchors; changing interpolation method from loess to spline')
-  }
-
-  if (is.list(anchors) && is.numeric(len) && nrow(anchors) > len) {
-    # if there are more anchors than len, pick just some anchors at random
-    anchors = as.data.frame(anchors)
-    idx = sample(1:nrow(anchors), size = len, replace = FALSE)
-    idx = sort(idx)
-    idx[1] = 1
-    idx[len] = nrow(anchors)  # make sure the first and last anchors are preserved
-    anchors = anchors[idx, ]
+  } else if (is.list(anchors) && nrow(anchors) > 50) {
+    interpol = 'approx'
   }
 
   if (!is.null(valueFloor)) {
@@ -105,64 +117,70 @@ getSmoothContour = function(anchors = data.frame(time = c(0, 1), value = c(0, 1)
     duration_ms = len / samplingRate * 1000
   }
 
-  if (!is.numeric(duration_ms) || duration_ms == 0 || !is.numeric(len) || len == 0) {
+  # if there are more anchors than len, pick just some anchors at random
+  if (nrow(anchors) > len) {
+    anchors = as.data.frame(anchors)
+    idx = sample(1:nrow(anchors), size = len, replace = FALSE)
+    idx = sort(idx)
+    idx[1] = 1
+    idx[len] = nrow(anchors)  # make sure the first and last anchors are preserved
+    anchors = anchors[idx, ]
+  }
+
+  if (!is.numeric(duration_ms) || duration_ms == 0 ||
+      !is.numeric(len) || len == 0) {
     return(NA)
   }
 
-  time = 1:len
-  if (nrow(anchors) == 1) {
-    # flat
-    smoothContour = rep(anchors$value[1], len)
-  } else if (nrow(anchors) == 2) {
-    # linear
-    smoothContour = seq(anchors$value[1], anchors$value[2], length.out = len)
+  if (discontThres <= 0 | nrow(anchors) < 3) {
+    smoothContour = drawContour(len = len,
+                                anchors = anchors,
+                                interpol = interpol,
+                                valueFloor = valueFloor,
+                                valueCeiling = valueCeiling,
+                                duration_ms = duration_ms)
   } else {
-    # smooth contour
-    if (method == 'spline') {
-      smoothContour = spline(anchors$value, n = len, x = anchors$time)$y
-      # plot(smoothContour, type='l')
-    } else if (method == 'loess') {
-      anchor_time_points = anchors$time - min(anchors$time)
-      anchor_time_points = anchor_time_points / max(anchor_time_points) * len
-      anchor_time_points[anchor_time_points == 0] = 1
-      anchors_long = as.vector(rep(NA, len))
-      anchors_long[anchor_time_points] = anchors$value # plot (anchors_long)
-
-      # let's draw a smooth curve through the given anchors
-      span = (1 / (1 + exp(duration_ms / 500)) + 0.5) / 1.1 ^ (nrow(anchors) - 3)
-      # NB: need to compensate for variable number of points, namely decrease
-      # smoothing as the number of points increases, hence the "1.1^..." term
-      # duration_ms = 50:9000
-      # span = 1 / (1 + exp(duration_ms / 500)) + 0.5
-      # plot(duration_ms, span, type = 'l')
-      l = suppressWarnings(loess(anchors_long ~ time, span = span))
-      # plot (time, anchors_long)
-      smoothContour = try (predict(l, time), silent = TRUE)
-      # plot(time, smoothContour)
-
-      # for long duration etc, larger span may be needed to avoid error in loess
-      while (class(smoothContour) == 'try-error') {
-        span = span + 0.1
-        l = suppressWarnings(loess(anchors_long ~ time, span = span))
-        smoothContour = try (predict(l, time), silent = TRUE)
-      }
-      # plot (smoothContour, type = 'l')
-
-      while (sum(smoothContour < valueFloor - 1e-6, na.rm = TRUE) > 0) {
-        # in case we get values below valueFloor, less smoothing should be used
-        # NB: -1e-6 avoids floating point problem, otherwise we get
-        # weird cases of -120 (float) < -120 (integer)
-        span = span / 1.1
-        l = suppressWarnings(loess(anchors_long ~ time, span = span))
-        smoothContour = try (predict(l, time), silent = TRUE)
+    # some anchors might be too close, so we split the contour into segments
+    # to avoid weird behavior of loess etc
+    sections = splitContour(anchors = anchors,
+                            discontThres = discontThres,
+                            jumpThres = jumpThres)
+    smoothContour = vector()
+    for (i in 1:nrow(sections)) {
+      segm_len = round((anchors$time[sections$end[i]] -
+                          anchors$time[sections$start[i]]) /
+                         diff(range(anchors$time)) * len)
+      cont = drawContour(len = segm_len,
+                         anchors = anchors[sections$start[i]:sections$end[i], ],
+                         interpol = interpol,
+                         valueFloor = valueFloor,
+                         valueCeiling = valueCeiling,
+                         duration_ms = duration_ms)
+      transition = vector()
+      if (i < nrow(sections)) {
+        trans_len = round((anchors$time[sections$start[i + 1]] -
+                             anchors$time[sections$end[i]]) /
+                            diff(range(anchors$time)) * len)
+        if (sections$jump[i] && length(cont) > 0) {
+          # upsample the segment before the jump to make up for skipped transition
+          cont = approx(cont, n = length(cont) + trans_len)$y
+        } else {
+          # make a linear transition preserving spacing between anchors
+          transition = seq(anchors$value[sections$end[i]],
+                           anchors$value[sections$start[i + 1]],
+                           length.out = trans_len)
+        }
       }
+      smoothContour = c(smoothContour, cont, transition)
+    }
+    if (length(smoothContour) != len) {
+      smoothContour = approx(smoothContour, n = len)$y
     }
-    smoothContour[smoothContour < valueFloor] = valueFloor
-    smoothContour[smoothContour > valueCeiling] = valueCeiling
   }
-  # plot(smoothContour, type='l')
+  # plot(smoothContour, type='p')
 
   if (plot) {
+    time = 1:len
     op = par("mar") # save user's original margin settings
     idx = seq(1, len, length.out = min(len, 100))
     # for plotting, shorten smoothContour to max 100 points
@@ -232,6 +250,123 @@ getSmoothContour = function(anchors = data.frame(time = c(0, 1), value = c(0, 1)
 }
 
 
+#' Draw contour
+#'
+#' Internal soundgen function
+#'
+#' The core part of getSmoothContour() that actually performs the interpolation
+#' between anchors.
+#' @inheritParams getSmoothContour
+#' @param duration_ms contour duration, ms
+#' @keywords internal
+drawContour = function(len,
+                       anchors,
+                       interpol,
+                       valueFloor,
+                       valueCeiling,
+                       duration_ms = 500) {
+  time = 1:len
+  if (nrow(anchors) == 1) {
+    # flat
+    smoothContour = rep(anchors$value[1], len)
+  } else if (nrow(anchors) == 2) {
+    # linear
+    smoothContour = seq(anchors$value[1], anchors$value[2], length.out = len)
+  } else {
+    # smooth contour
+    if (interpol == 'approx') {
+      smoothContour = approx(anchors$value, n = len, x = anchors$time)$y
+      # plot(smoothContour, type='l')
+    } else if (interpol == 'spline') {
+      smoothContour = spline(anchors$value, n = len, x = anchors$time)$y
+      # plot(smoothContour, type='l')
+    } else if (interpol == 'loess') {
+      anchor_time_points = anchors$time - min(anchors$time)
+      anchor_time_points = anchor_time_points / max(anchor_time_points) * len
+      anchor_time_points[anchor_time_points == 0] = 1
+      anchors_long = as.vector(rep(NA, len))
+      anchors_long[anchor_time_points] = anchors$value # plot (anchors_long)
+
+      # let's draw a smooth curve through the given anchors
+      span = (1 / (1 + exp(duration_ms / 500)) + 0.5) / 1.1 ^ (nrow(anchors) - 3)
+      # NB: need to compensate for variable number of points, namely decrease
+      # smoothing as the number of points increases, hence the "1.1^..." term
+      # duration_ms = 50:9000
+      # span = 1 / (1 + exp(duration_ms / 500)) + 0.5
+      # plot(duration_ms, span, type = 'l')
+      l = suppressWarnings(loess(anchors_long ~ time, span = span))
+      # plot (time, anchors_long)
+      smoothContour = try (predict(l, time), silent = TRUE)
+      # plot(time, smoothContour)
+
+      # for long duration etc, larger span may be needed to avoid error in loess
+      while (class(smoothContour) == 'try-error') {
+        span = span + 0.1
+        l = suppressWarnings(loess(anchors_long ~ time, span = span))
+        smoothContour = try (predict(l, time), silent = TRUE)
+      }
+      # plot (smoothContour, type = 'l')
+
+      while (sum(smoothContour < valueFloor - 1e-6, na.rm = TRUE) > 0) {
+        # in case we get values below valueFloor, less smoothing should be used
+        # NB: -1e-6 avoids floating point problem, otherwise we get
+        # weird cases of -120 (float) < -120 (integer)
+        span = span / 1.1
+        l = suppressWarnings(loess(anchors_long ~ time, span = span))
+        smoothContour = try (predict(l, time), silent = TRUE)
+      }
+    }
+    smoothContour[smoothContour < valueFloor] = valueFloor
+    smoothContour[smoothContour > valueCeiling] = valueCeiling
+    return(smoothContour)
+  }
+}
+
+#' Split contour
+#'
+#' Internal soundgen function
+#'
+#' Splits a smooth contour into several segments. A new segments is started if
+#' the time step between two anchors is smaller than discontThres.
+#' @param anchors a dataframe with two columns: time and value (time on any scale)
+#' @inheritParams getSmoothContour
+#' @return Returns a dataframe containing the index of anchor rows for start and
+#'   end of each segment and whether we want a transition or a jump between
+#'   segments.
+#' @keywords internal
+#' @examples
+#' soundgen:::splitContour(data.frame(time = c(0, 370, 650, 655, 1050, 1400),
+#'   value = c(360, 316, 345, 550, 610, 590)))
+#' soundgen:::splitContour(data.frame(time = c(0, .2, .205, .8, .81, 1),
+#'   value = c(360, 316, 345, 550, 610, 590)))
+#' soundgen:::splitContour(data.frame(time = c(0, .4, .45, .6, .8, 1),
+#'   value = c(360, 316, 345, 550, 610, 590)))
+#' soundgen:::splitContour(data.frame(time = c(0, .4, .45, .6, .8, 1),
+#'   value = c(360, 316, 345, 550, 610, 590)),
+#'   discontThres = .1)
+#' soundgen:::splitContour(data.frame(time = c(0, 1),
+#'   value = c(360, 590)))
+splitContour = function(anchors,
+                        discontThres = .05,
+                        jumpThres = .01) {
+  discont = which(diff(anchors$time) / diff(range(anchors$time)) < discontThres)
+  jumps = which(diff(anchors$time) / diff(range(anchors$time)) < jumpThres)
+  if (length(discont) > 0) {
+    sections = data.frame(start = 1,
+                          end = rep(nrow(anchors), length(discont) + 1),
+                          jump = FALSE)
+    for (j in 1:length(discont)) {
+      sections$end[j] = discont[j]
+      sections$start[j + 1] = discont[j] + 1
+      sections$jump[j] = discont[j] %in% jumps
+    }
+  } else {
+    sections = data.frame(start = 1, end = nrow(anchors), jump = FALSE)
+  }
+  return(sections)
+}
+
+
 #' Discrete smooth contour from anchors
 #'
 #' Internal soundgen function.
@@ -246,12 +381,12 @@ getSmoothContour = function(anchors = data.frame(time = c(0, 1), value = c(0, 1)
 #' @keywords internal
 #' @examples
 #' # for a bout consisting of 10 syllables
-#' soundgen:::getDiscreteContour(len = 10, method = 'spline', plot = TRUE,
+#' soundgen:::getDiscreteContour(len = 10, interpol = 'spline', plot = TRUE,
 #'   ylab = 'Semitones', anchors = data.frame(time = c(0, .2, .6, 1),
 #'   value = c(0, -3, 1, 0)))
 getDiscreteContour = function(len,
                               anchors = data.frame(time = c(0, 1), value = c(1, 1)),
-                              method = c('spline', 'loess')[2],
+                              interpol = c('spline', 'loess')[2],
                               valueFloor = NULL,
                               valueCeiling = NULL,
                               ylim = NULL,
@@ -260,7 +395,7 @@ getDiscreteContour = function(len,
   contour = getSmoothContour(
     len = len,
     anchors = anchors,
-    method = method,
+    interpol = interpol,
     valueFloor = valueFloor,
     valueCeiling = valueCeiling
   )
@@ -287,11 +422,13 @@ getDiscreteContour = function(len,
 #' therefore processed directly in soundgen()
 #' @param anchors a numeric vector of values or a list/dataframe with one column
 #'   (value) or two columns (time and value)
+#' @param normalizeTime if TRUE, normalizes anchors$time values to range from 0 to 1
+#' @keywords internal
 #' @examples
 #' soundgen:::reformatAnchors(150)
 #' soundgen:::reformatAnchors(c(150, 200, 220))
 #' soundgen:::reformatAnchors(anchors = list(value=c(150, 200, 220)))
-#' soundgen:::reformatAnchors(anchors = list(time = c(0, .1, 1),
+#' soundgen:::reformatAnchors(anchors = list(time = c(0, 10, 100),
 #'                                           value = c(150, 200, 220)))
 
 #' # returns NA
@@ -299,7 +436,12 @@ getDiscreteContour = function(len,
 #' # returns NA with a warning
 #' soundgen:::reformatAnchors(anchors = list(time = c(0, .1, 1),
 #'                                           freq = c(150, 200, 220)))
-reformatAnchors = function(anchors) {
+#' \dontrun{
+#' # throws an error
+#' soundgen:::reformatAnchors(anchors = list(time = c(0, .8, .7, 1),
+#'                                           value = c(150, 200, 150, 220)))
+#' }
+reformatAnchors = function(anchors, normalizeTime = TRUE) {
   if (is.numeric(anchors)) {
     # for numeric vectors, assume these are equally spaced anchor values
     anchors_df = data.frame(
@@ -325,5 +467,14 @@ reformatAnchors = function(anchors) {
   } else {
     return(NA)
   }
+
+  # make sure time values are in the right order
+  if (any(diff(anchors_df$time) < 0)) {
+    stop('Time stamps of anchors must increase monotonically')
+  }
+
+  # make sure time ranges from 0 to 1
+  if (normalizeTime) anchors_df$time = zeroOne(anchors_df$time)
+
   return(anchors_df)
 }
